using System;
using System.Collections.Generic;
using System.Text;

namespace LD10Tanks
{
    /// <summary>
    /// A bit opened List<> implementation, to allow for raw inside access
    /// and to be sure the list only grows.
    /// </summary>
    public class FastList<T>
    {
        private T[] m_list;
        private int m_size;

        public T[] RawList
        {
            get { return m_list; }
        }

        public int Count
        {
            get { return m_size; }
        }

        public T this [int idx]
        {
            get { return m_list[idx]; }
            set { m_list[idx] = value; }
        }

        public FastList()
            : this(16)
        {
        }

        public FastList(int capacity)
        {
            m_list = new T[capacity];
        }

        public void Clear()
        {
            // Do not clear the m_list elements; may lead to pointer leaks but it's faster

            m_size = 0;
        }

        public void Add(T val)
        {
            if (m_size >= m_list.Length)
            {
                int new_size = (m_list.Length + 16) * 2;
                T[] new_list = new T[new_size];

                Array.Copy(m_list, new_list, m_list.Length);

                m_list = new_list;
            }

            m_list[m_size++] = val;
        }

        public void CopyFrom(FastList<T> lst)
        {
            if (m_list.Length < lst.m_size)
            { // Allocate enough memory
                m_list = new T[lst.m_size];
            }

            // Copy the data
            Array.Copy(lst.m_list, m_list, lst.m_size);
            m_size = lst.m_size;
        }

        public T[] ToArray()
        {
            T[] ret = new T[m_size];
            Array.Copy(m_list, ret, m_size);
            return ret;
        }
    }
}
