using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

namespace LD10Tanks
{
    public enum WallType
    {
        Normal,
        Tall,
        HoleShallow,
        HoleDeep,
    }

    public enum TankType
    {
        Gun1,
        Gun2,
        Gun3,
        Gun4,
        Gun5,
    }


    public class Tank
    {
        public Vector2 pos;
        public bool fixed_pos;
        public int bullets;
        public TankType type;

        public Vector3 look_dir;
        public int fired;

        public Vector3 Pos3D
        {
            get { return new Vector3(pos.X, 0, pos.Y); }
        }

        public Vector3 GunPos3D
        {
            get { return new Vector3(pos.X, TankMeshes.gun_height, pos.Y); }
        }

        public Matrix BaseMatrix
        {
            get { return Matrix.CreateTranslation(Pos3D);  }
        }

        public Matrix TopMatrix
        {
            get { return Matrix.Transpose(Matrix.CreateLookAt(Vector3.Zero, look_dir, Vector3.UnitY)) * BaseMatrix; }
        }

        public int GunCount
        {
            get { return 1 + (int)type; }
        }


        public Tank()
        {
            float ang = (float)(GameMain.random.NextDouble() * Math.PI * 2);

            look_dir = new Vector3((float)Math.Cos(ang), 0, (float)Math.Sin(ang));
        }

        /// <summary>
        /// Shallow copy; designed to make a new game object.
        /// </summary>
        public Tank(Tank src)
            : this()
        {
            pos = src.pos;
            fixed_pos = src.fixed_pos;
            bullets = src.bullets;
            type = src.type;

            look_dir = src.look_dir;
        }

        /// <summary>
        /// Returns the rotated gun direction.
        /// </summary>
        public Vector3 GetGunDir(int idx)
        {
            Matrix mtx = TopMatrix;
            Vector3 base_dir = TankMeshes.gun_dirs[(int)type][idx];

            Vector4 ret = Vector4.Transform(new Vector4(base_dir, 0), mtx);
            return new Vector3(ret.X, 0, ret.Z);
        }

        public void Reset()
        {
            fired = 0;
        }
    }

    public class Wall
    {
        public Vector2 pos1;
        public Vector2 pos2;
        public float thickness;
        public WallType type;

        // Collision information
        public Vector2 coll_center;
        public Vector2 coll_dir1;
        public Vector2 coll_dir2;
        public Vector2 coll_extents;

        public void PrecalcCollisions()
        {
            coll_center = (pos1 + pos2) * 0.5f;
            coll_dir1 = pos2 - pos1;
            coll_dir1.Normalize();
            coll_dir2 = new Vector2(-coll_dir1.Y, coll_dir1.X);
            coll_extents = new Vector2((pos2 - pos1).Length(), thickness) * 0.5f;
        }
    }

    public class GameLevel
    {
        public String name = "";
        public int unique_id;

        public List<Tank> tanks = new List<Tank>();
        public List<Wall> walls = new List<Wall>();

        public bool BallIntersectsWalls(Vector2 pos, float radius)
        {
            float rad2 = radius * radius;

            for (int i = 0; i < walls.Count; ++i)
            {
                Wall wall = walls[i];

                Vector2 to_sphere = pos - wall.coll_center;
                float sx = Vector2.Dot(to_sphere, wall.coll_dir1);
                float sy = Vector2.Dot(to_sphere, wall.coll_dir2);

                float bx = MathHelper.Clamp(sx, -wall.coll_extents.X, wall.coll_extents.X);
                float by = MathHelper.Clamp(sy, -wall.coll_extents.Y, wall.coll_extents.Y);

                float diff_x = sx - bx;
                float diff_y = sy - by;
                float dist_sq = diff_x * diff_x + diff_y * diff_y;
                if (dist_sq <= rad2)
                    return true;
            }

            return false;
        }
    }
}
