using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Text;
using System.Xml;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Storage;

namespace LD10Tanks
{
    public class GameLevelList
    {
        public List<GameLevel> levels = new List<GameLevel>();

        public GameLevelList()
        {
            // Load the XML
            XmlDocument xml = new XmlDocument();

            // Load from embedded resource
            /*using (Stream stm = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream("LD10Tanks.Content.levels.xml"))
            {
                xml.Load(stm);
                stm.Close();
            }*/

            // Load from file
            xml.Load(Path.Combine(Path.Combine(StorageContainer.TitleLocation, "Content"), "levels.xml"));

            // Load the levels
            foreach (XmlElement level_node in xml["levels"].SelectNodes("level"))
            {
                GameLevel level = new GameLevel();
                levels.Add(level);

                level.name = GetStringValue(level_node["name"], level.name);
                level.unique_id = GetIntValue(level_node["unique_id"], level.unique_id);

                foreach (XmlElement node in level_node.SelectNodes("tank"))
                {
                    Tank tank = new Tank();
                    level.tanks.Add(tank);

                    tank.pos.X = GetFloatValue(node["x"], 0) - 5.0f;
                    tank.pos.Y = GetFloatValue(node["y"], 0) - 5.0f;
                    tank.fixed_pos = GetIntValue(node["fixed_position"], 0) != 0;
                    tank.bullets = GetIntValue(node["bullets"], tank.bullets);
                    tank.type = (TankType)GetIntValue(node["type"], 0);
                }

                foreach (XmlElement node in level_node.SelectNodes("wall"))
                {
                    Wall wall = new Wall();
                    level.walls.Add(wall);

                    wall.pos1.X = GetFloatValue(node["x1"], 0) - 5.0f;
                    wall.pos1.Y = GetFloatValue(node["y1"], 0) - 5.0f;
                    wall.pos2.X = GetFloatValue(node["x2"], 0) - 5.0f;
                    wall.pos2.Y = GetFloatValue(node["y2"], 0) - 5.0f;
                    wall.thickness = GetFloatValue(node["thickness"], 0);
                    wall.type = (WallType)GetIntValue(node["type"], 0);

                    // Prepare for collisions
                    wall.PrecalcCollisions();
                }
            }
        }

        private string GetStringValue(XmlElement node, string def)
        {
            if (node == null)
                return def;
            string ret = node.GetAttribute("value");
            if (ret == null)
                return def;
            return ret;
        }

        private int GetIntValue(XmlElement node, int def)
        {
            try
            {
                return Int32.Parse(GetStringValue(node, null));
            }
            catch
            {
                return def;
            }
        }

        private float GetFloatValue(XmlElement node, float def)
        {
            int def_int = (int)(def * 65536.0f);
            int ret = GetIntValue(node, def_int);
            return (float)ret / 65536.0f;
        }
    }
}
