using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace LD10Tanks
{
    public class ParticleManager
    {
        private const int MaxParticles = 10000;

        private struct Particle
        {
            public Vector3 pos;
            public Vector3 vel;
            public Color color;
            public float lifetime;
            public float fade_start_lifetime;
        }

        public VertexPositionColor[] vertices = new VertexPositionColor[MaxParticles];
        public int vertex_count;

        private Particle[] m_particles = new Particle[MaxParticles];
        private int m_particle_cnt;

        public void Update(float delta_t)
        {
            // Update every particle
            int write_pos = 0;
            for (int read_pos = 0; read_pos < m_particle_cnt; ++read_pos)
            {
                Particle p = m_particles[read_pos];

                p.pos += p.vel * delta_t;
                p.lifetime -= delta_t;

                Color c = p.color;

                // Fading out
                if (p.lifetime < p.fade_start_lifetime)
                {
                    byte alpha = (byte) ((float)c.A * (p.lifetime / p.fade_start_lifetime));
                    c = new Color(c.R, c.G, c.B, alpha);
                }

                // Keep only the particles who are still alive
                if (p.lifetime > 0.0f)
                {
                    vertices[write_pos] = new VertexPositionColor(p.pos, c);
                    m_particles[write_pos++] = p;
                }
            }
            m_particle_cnt = write_pos;
            vertex_count = write_pos;
        }

        public void SpawnParticles(Vector3 pos, float radius, float speed, float lifetime, int amount, Color color)
        {
            while (amount-- > 0)
            {
                if (m_particle_cnt >= m_particles.Length)
                    return;

                Particle p;
                p.pos = pos + RandUnitVector3() * RandFloat(0, radius);
                p.vel = RandUnitVector3() * (speed * RandFloat(0.9f, 1.1f));
                p.color = color;
                p.lifetime = lifetime * RandFloat(0.5f, 1.5f);
                p.fade_start_lifetime = p.lifetime * 0.2f;

                m_particles[m_particle_cnt++] = p;
            }
        }

        private static Vector3 RandUnitVector3()
        {
            Vector3 v;
            do
            {
                v.X = RandFloat();
                v.Y = RandFloat();
                v.Z = RandFloat();
            } while (v.LengthSquared() < 0.01f);
            v.Normalize();
            return v;
        }

        /// <summary>
        /// -1 .. 1
        /// </summary>
        private static float RandFloat()
        {
            return RandFloat(-1, 1);
        }

        private static float RandFloat(float min, float max)
        {
            return min + (float)GameMain.random.NextDouble() * (max - min);
        }
    }
}
