using System;
using System.Collections.Generic;
using System.Drawing;
using System.Text;

namespace TankLevelEditor
{
    public enum WallType
    {
        Normal,
        Tall,
        HoleShallow,
        HoleDeep,
    }

    public abstract class LevelItem
    {
        public abstract int PointCount { get; }

        public abstract PointF GetPoint(int idx);
        public abstract void SetPoint(int idx, PointF pt);
    }

    public class Wall : LevelItem
    {
        private float m_thickness = 0.5f;
        public float Thickness
        {
            get { return m_thickness; }
            set { m_thickness = value; }
        }

        private WallType m_type = WallType.Normal;
        public WallType Type
        {
            get { return m_type; }
            set { m_type = value; }
        }

        public PointF pt1 = new PointF(5, 4);
        public PointF pt2 = new PointF(5, 6);

        public override int PointCount { get { return 2; } }

        public override PointF GetPoint(int idx)
        {
            return (idx == 1) ? pt2 : pt1;
        }

        public override void SetPoint(int idx, PointF pt)
        {
            if (idx == 1)
                pt2 = pt;
            else
                pt1 = pt;
        }
    }

    public enum TankType
    {
        Gun1,
        Gun2,
        Gun3,
        Gun4,
        Gun5,
    }

    public class Tank : LevelItem
    {
        private bool m_fixed_position = false;
        public bool FixedPosition
        {
            get { return m_fixed_position; }
            set { m_fixed_position = value; }
        }

        private TankType m_type = TankType.Gun1;
        public TankType Type
        {
            get { return m_type; }
            set { m_type = value; }
        }

        private int m_bullets = 1;
        public int Bullets
        {
            get { return m_bullets; }
            set { m_bullets = Math.Max(value, 1); }
        }

        public PointF pos = new PointF(5, 5);

        public override int PointCount { get { return 1; } }

        public override PointF GetPoint(int idx)
        {
            return pos;
        }

        public override void SetPoint(int idx, PointF pt)
        {
            pos = pt;
        }
    }

    public class Level
    {
        private static Random m_random = new Random();

        private string m_name = "Untitled";
        public string Name
        {
            get { return m_name; }
            set { m_name = value; }
        }

        private int m_unique_id = m_random.Next();
        public int UniqueId
        {
            get { return m_unique_id; }
            set { m_unique_id = value; }
        }

        public List<Wall> walls = new List<Wall>();
        public List<Tank> tanks = new List<Tank>();
    }
}
